import 'dart:html';

import 'package:flutter/material.dart';
import 'package:flutter_markdown/flutter_markdown.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:remove_emoji/remove_emoji.dart';
import 'package:markdown/markdown.dart' as md;
import 'dart:ui' as ui;
import 'package:url_launcher/url_launcher.dart';

abstract class Parser {
  Widget parse(Map block, {Options options, BuildContext context});
}

class Options {
  const Options({this.margin = const EdgeInsets.all(0)});

  final EdgeInsets margin;
}

const defaultButtonParser = _ButtonParser();
const defaultDividerParser = _DividerParser();
const defaultHeaderParser = _HeaderParser();
const defaultImageParser = _ImageParser();
const defaultSectionParser = _SectionParser();
const defaultSpacerParser = _SpacerParser();

class _ButtonParser implements Parser {
  const _ButtonParser();

  @override
  Widget parse(Map block, {Options options = const Options(), BuildContext context}) {
    //var action = block["action"] != null ? block["action"] : {};
    var text = block["text"];
    var hasEmoji = text["emoji"];
    var innerType = text["type"];
    var innerText = text["text"];
    if (hasEmoji != null && hasEmoji == false) {
      innerText = RemoveEmoji().removemoji(innerText);
    }

    Widget inner = Container();
    if (innerType == "plain_text") {
      inner = Container(
        child: Text(innerText,
            style: GoogleFonts.getFont('Open Sans', fontSize: 14, fontWeight: FontWeight.w600, color: Colors.black)),
        padding: EdgeInsets.fromLTRB(8, 6, 8, 6),
        decoration: BoxDecoration(
            border: Border.all(width: 0.5, color: Colors.grey.shade600),
            borderRadius: BorderRadius.all(Radius.circular(4))),
      );
    } else if (innerType == "mrkdwn") {
      inner = MarkdownBody(
        data: innerText,
        extensionSet: md.ExtensionSet(
          md.ExtensionSet.gitHubFlavored.blockSyntaxes,
          [md.EmojiSyntax(), ...md.ExtensionSet.gitHubFlavored.inlineSyntaxes],
        ),
        onTapLink: (text, href, title) {
          launch(href);
        },
      );
    }

    return Container(
        margin: options.margin,
        child: TextButton(
          onPressed: () {},
          child: inner,
          style: ButtonStyle(overlayColor: MaterialStateProperty.all(Colors.grey.shade100)),
        ));
  }
}

class _SpacerParser implements Parser {
  const _SpacerParser();

  @override
  Widget parse(Map block, {Options options = const Options(), BuildContext context}) {
    return Spacer();
  }
}

class _SectionParser implements Parser {
  const _SectionParser();

  @override
  Widget parse(Map block, {Options options = const Options(), BuildContext context}) {
    var tb = block["text"];
    var text = tb["text"];

    if (tb != null) {
      var hasEmoji = tb["emoji"];
      if (hasEmoji != null && hasEmoji == false) {
        text = RemoveEmoji().removemoji(text);
      }
      if (tb["type"] == "plain_text") {
        return Text(
          text,
          textAlign: TextAlign.left,
          style: GoogleFonts.getFont('Open Sans', fontSize: 14, fontWeight: FontWeight.w400, color: Colors.black),
        );
      } else if (tb["type"] == "mrkdwn") {
        return Container(
          child: MarkdownBody(
            data: text,
            extensionSet: md.ExtensionSet(
              md.ExtensionSet.gitHubFlavored.blockSyntaxes,
              [md.EmojiSyntax(), ...md.ExtensionSet.gitHubFlavored.inlineSyntaxes],
            ),
            styleSheet: MarkdownStyleSheet.fromTheme(Theme.of(context)).copyWith(
                p: GoogleFonts.getFont('Open Sans',
                    fontSize: 14, fontWeight: FontWeight.w400, color: Colors.black, height: 1.4)),
            onTapLink: (text, href, title) {
              launch(href);
            },
          ),
        );
      }
    }
    return Container();
  }
}

class _DividerParser implements Parser {
  const _DividerParser();

  @override
  Widget parse(Map block, {Options options = const Options(), BuildContext context}) {
    return Divider(
      color: Colors.grey.shade500,
      height: 12,
      thickness: 0.5,
    );
  }
}

class _HeaderParser implements Parser {
  const _HeaderParser();

  @override
  Widget parse(Map block, {Options options = const Options(), BuildContext context}) {
    var text = block["text"];
    var innerType = text["type"];
    var innerText = text["text"] != null ? text["text"] : "";
    var hasEmoji = text["emoji"];
    if (hasEmoji != null && hasEmoji == false) {
      innerText = RemoveEmoji().removemoji(innerText);
    }

    Widget inner = Container();
    if (innerType == "plain_text") {
      inner = Container(
          alignment: Alignment.centerLeft,
          padding: EdgeInsets.fromLTRB(0, 8, 0, 4),
          child: Text(innerText,
              style:
                  GoogleFonts.getFont('Open Sans', fontSize: 18, fontWeight: FontWeight.w700, color: Colors.black87)));
    } else if (innerType == "mrkdwn") {
      inner = MarkdownBody(
        data: innerText,
        extensionSet: md.ExtensionSet(
          md.ExtensionSet.gitHubFlavored.blockSyntaxes,
          [md.EmojiSyntax(), ...md.ExtensionSet.gitHubFlavored.inlineSyntaxes],
        ),
        onTapLink: (text, href, title) {
          launch(href);
        },
      );
    }

    return inner;
  }
}

class _ImageParser implements Parser {
  const _ImageParser();

  @override
  Widget parse(Map block, {Options options = const Options(), BuildContext context}) {
    var title = block["title"];
    var url = block["image_url"];
    var innerType = title != null ? title["type"] : "";

    var _image = ImageElement(src: url);
    // ignore: undefined_prefixed_name
    ui.platformViewRegistry.registerViewFactory(
      "_image",
      (int viewId) => _image,
    );

    Widget inner = Container();
    if (innerType == "plain_text") {
      var header = title["text"] != null ? title["text"] : "";

      inner = Container(
          alignment: Alignment.centerLeft,
          padding: EdgeInsets.fromLTRB(0, 8, 0, 4),
          child: Text(header,
              style: GoogleFonts.getFont('Open Sans',
                  fontSize: 14, fontWeight: FontWeight.w400, color: Colors.grey.shade600)));
    } else if (innerType == "mrkdwn") {
      inner = MarkdownBody(
        data: title["text"],
        extensionSet: md.ExtensionSet(
          md.ExtensionSet.gitHubFlavored.blockSyntaxes,
          [md.EmojiSyntax(), ...md.ExtensionSet.gitHubFlavored.inlineSyntaxes],
        ),
        onTapLink: (text, href, title) {
          launch(href);
        },
      );
    }

    return Container(
        child: Column(
      children: [
        inner,
        Container(
            alignment: Alignment.centerLeft,
            child: SizedBox(
                width: _image.width.toDouble(),
                height: _image.height.toDouble(),
                child: HtmlElementView(viewType: "_image"))),
      ],
    ));
  }
}
